@@ -12,16 +12,19 @@
 package org.eclipse.jst.j2ee.ui.project.facet;
 
 import org.eclipse.jface.wizard.WizardDialog;
+import org.eclipse.jst.j2ee.project.facet.J2EEModuleFacetInstallDataModelProvider;
 import org.eclipse.jst.j2ee.web.project.facet.IWebFacetInstallDataModelProperties;
 import org.eclipse.osgi.util.NLS;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.SelectionAdapter;
 import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.layout.GridData;
+import org.eclipse.swt.layout.GridLayout;
 import org.eclipse.swt.widgets.Button;
 import org.eclipse.swt.widgets.Combo;
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Control;
+import org.eclipse.swt.widgets.Group;
 import org.eclipse.swt.widgets.Label;
 import org.eclipse.wst.common.frameworks.datamodel.IDataModel;
 import org.eclipse.wst.common.frameworks.internal.datamodel.ui.DataModelSynchHelper;
@@ -42,27 +45,27 @@ public final class EarSelectionPanel implements IWebFacetInstallDataModelPropert
     private final IDataModel model;
     private DataModelSynchHelper synchhelper;
     
-  
-    
     public EarSelectionPanel( final IDataModel model, final Composite parent) 
     {
         this.model = model;
         this.synchhelper = new DataModelSynchHelper(model);
         
-        this.addToEar = new Button( parent, SWT.CHECK );
+        final Group group = new Group( parent, SWT.NONE );
+        group.setLayoutData( gdhfill() );
+        group.setLayout( new GridLayout( 3, false ) );
+        group.setText( Resources.earMemberShip );
+        
+        this.addToEar = new Button( group, SWT.CHECK );
         this.addToEar.setText( Resources.addToEarLabel );
         this.addToEar.setLayoutData( gdhspan( gdhfill(), 3 ) );
         synchhelper.synchCheckbox(addToEar, ADD_TO_EAR, null);
 
-        label = new Label(parent, SWT.NULL);
+        label = new Label(group, SWT.NULL);
         label.setText(Resources.earProjectLabel);
-        GridData gridData = new GridData();
-        gridData.horizontalIndent = 20;
-        label.setLayoutData(gridData);
-        this.combo = new Combo(parent, SWT.NONE);
+        this.combo = new Combo(group, SWT.NONE);
         this.combo.setLayoutData( gdhfill() );
         
-        this.newButton = new Button( parent, SWT.PUSH );
+        this.newButton = new Button( group, SWT.PUSH );
         this.newButton.setText( Resources.newButtonLabel );
         
         this.newButton.addSelectionListener( new SelectionAdapter()
@@ -85,7 +88,9 @@ public final class EarSelectionPanel implements IWebFacetInstallDataModelPropert
             = new WizardDialog( newButton.getShell(), wizard );
         
         IRuntime runtime = (IRuntime)model.getProperty(FACET_RUNTIME);
+        String earName = model.getStringProperty( J2EEModuleFacetInstallDataModelProvider.EAR_PROJECT_NAME );
     	wizard.setRuntimeInDataModel(runtime);
+    	wizard.setEARName( earName );
         if( dialog.open() != SWT.CANCEL )
         {
             model.notifyPropertyChange(EAR_PROJECT_NAME, IDataModel.VALID_VALUES_CHG);
@@ -114,6 +119,7 @@ public final class EarSelectionPanel implements IWebFacetInstallDataModelPropert
         public static String addToEarLabel;
         public static String newButtonLabel;
         public static String earProjectLabel;
+        public static String earMemberShip;
         
         static
         {
@@ -131,6 +137,5 @@ public final class EarSelectionPanel implements IWebFacetInstallDataModelPropert
     
     public String getComboText(){
     	return combo.getText();
-    }
-    
+    }    
 }
