@@ -1,18 +1,32 @@
 /*******************************************************************************
- * Copyright (c) 2004 IBM Corporation and others.
+ * Copyright (c) 2004, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
- *     IBM Corporation - initial API and implementation
+ * IBM Corporation - initial API and implementation
+ * yyyymmdd bug      Email and other contact information
+ * -------- -------- -----------------------------------------------------------
+ * 20060410   135441 joan@ca.ibm.com - Joan Haggarty
+ * 20060410   136011 kathy@ca.ibm.com - Kathy Chan
+ * 20060420   135912 joan@ca.ibm.com - Joan Haggarty
+ * 20060504   119296 pmoogk@ca.ibm.com - Peter Moogk
+ * 20060525   142281 joan@ca.ibm.com - Joan Haggarty
+ * 20060607   144932 kathy@ca.ibm.com - Kathy Chan
+ * 20060612   145081 pmoogk@ca.ibm.com - Peter Moogk
+ * 20060719   139977 kathy@ca.ibm.com - Kathy Chan
+ * 20060803   152701 cbrealey@ca.ibm.com - Chris Brealey
+ * 20060803   152486 makandre@ca.ibm.com - Andrew Mak, Typing WSDL in Service definition field is very slow
+ * 20060825   135570 makandre@ca.ibm.com - Andrew Mak, Service implementation URL not displayed properly on first page
+ * 20060831   155883 makandre@ca.ibm.com - Andrew Mak, service impl field doesn't work after switching from BU to TD
+ * 20061211   161589 makandre@ca.ibm.com - Andrew Mak, NPE in service generation after opening and cancelling from browse dialog
  *******************************************************************************/
 package org.eclipse.jst.ws.internal.consumption.ui.widgets.object;
 
 import java.io.File;
 import java.net.MalformedURLException;
-
 import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IResource;
@@ -31,9 +45,10 @@ import org.eclipse.jface.viewers.StructuredSelection;
 import org.eclipse.jst.j2ee.webservice.wsclient.ServiceRef;
 import org.eclipse.jst.ws.internal.common.J2EEActionAdapterFactory;
 import org.eclipse.jst.ws.internal.common.ResourceUtils;
+import org.eclipse.jst.ws.internal.common.UniversalPathTransformer;
 import org.eclipse.jst.ws.internal.consumption.common.WSDLParserFactory;
 import org.eclipse.jst.ws.internal.consumption.ui.ConsumptionUIMessages;
-import org.eclipse.jst.ws.internal.consumption.ui.widgets.WSDLSelectionConditionCommand;
+import org.eclipse.jst.ws.internal.consumption.ui.widgets.TimedWSDLSelectionConditionCommand;
 import org.eclipse.jst.ws.internal.consumption.ui.widgets.WSDLSelectionTreeWidget;
 import org.eclipse.jst.ws.internal.ui.common.DialogResourceBrowser;
 import org.eclipse.jst.ws.internal.ui.common.FileExtensionFilter;
@@ -44,6 +59,7 @@ import org.eclipse.swt.events.ModifyEvent;
 import org.eclipse.swt.events.ModifyListener;
 import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.events.SelectionListener;
+import org.eclipse.swt.graphics.Point;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.widgets.Button;
 import org.eclipse.swt.widgets.Composite;
@@ -68,11 +84,17 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
   private String              pluginId_;
   private FileExtensionFilter wsFilter_;
   private WebServicesParser webServicesParser;
-
+  private String wsdlURI_;
+  
   private Composite parent_;
   private Listener  statusListener_;
+  private ModifyListener modifyListener_;
   private WSDLSelectionTreeWidget tree;
   
+  private Timer timer_ = null;
+  
+  private UniversalPathTransformer transformer_ = new UniversalPathTransformer();
+  
   /*CONTEXT_ID PCON0001 for the WSDL Selection Page*/
   private final String INFOPOP_PCON_PAGE = "PCON0001";
 
@@ -92,7 +114,10 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
   /*CONTEXT_ID PCON0005 for the Wizard WSDL Validation summary message of the WSDL Selection Page*/
   private Text validationSummaryText_;
   private Text validationSummaryText2_;
-
+  
+  /*CONTEXT_ID PCON0006 for the Stop Wizard WSDL Validation button of the WSDL Selection Page*/
+  private Button stopValidationButton_;
+  private final String INFOPOP_PCON_BUTTON_STOP_VALIDATION = "PCON0006";
   
   public WSDLSelectionWidget()
   {
@@ -106,6 +131,7 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
       {
 		  msgViewer_.setInput(validateWSDLJob_.getValidationMessages());
 		  updateValidationSummary(validateWSDLJob_.getValidationMessageSeverity());
+		  setValidationInProgress(false);
       }
     };
     
@@ -141,14 +167,17 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
     webServiceURI = uiUtils.createText( wsdlGroup, null, 
     						ConsumptionUIMessages.TOOLTIP_PCON_TEXT_WS, 
     						INFOPOP_PCON_TEXT_WSDL, SWT.SINGLE | SWT.BORDER );
-    webServiceURI.addModifyListener(
+    modifyListener_ =
       new ModifyListener()
       {
         public void modifyText(ModifyEvent event)
         {
           handleWebServiceURIModifyEvent();
         }
-      });
+      };
+    
+    webServiceURI.addModifyListener(modifyListener_);
+    
 //    webServiceURI.addListener( SWT.Modify, statusListener );
 
     wsBrowseButton_ = uiUtils.createPushButton( wsdlGroup, ConsumptionUIMessages.BUTTON_BROWSE, 
@@ -170,14 +199,14 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
 
     tree = new WSDLSelectionTreeWidget();
     tree.addControls(parent, statusListener);
-    tree.setWebServicesParser(webServicesParser);
+    tree.setWebServicesParser(webServicesParser);    
     
     msgViewer_ = new ValidationMessageViewerWidget();
     msgViewer_.addControls(parent, statusListener);
     
     validationSummaryText_ = new Text( parent, SWT.WRAP);
     validationSummaryText_.setEditable(false);
-    GridData gd1 = new GridData(GridData.FILL_BOTH);
+    GridData gd1 = new GridData(SWT.FILL, SWT.FILL, false, false);
     validationSummaryText_.setLayoutData(gd1);
     validationSummaryText_.setToolTipText( ConsumptionUIMessages.TOOLTIP_VALIDATE_TEXT_MESSAGE_SUMMARY );
     
@@ -185,6 +214,26 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
     validationSummaryText2_.setEditable(false);
     validationSummaryText2_.setLayoutData(gd1);
     
+    stopValidationButton_ = uiUtils.createPushButton(parent, 
+    			ConsumptionUIMessages.LABEL_BUTTON_STOP_WSDL_VALIDATION, 
+    			ConsumptionUIMessages.TOOLTIP_STOP_VALIDATION_BUTTON, 
+    			INFOPOP_PCON_BUTTON_STOP_VALIDATION);
+    setValidationInProgress(false);
+    
+    stopValidationButton_.addSelectionListener(
+    		new SelectionListener()
+    		{
+    			public void widgetDefaultSelected(SelectionEvent event)
+    			{
+    				handleStopValidationButton();
+    			}
+
+    			public void widgetSelected(SelectionEvent event)
+    			{
+    				handleStopValidationButton();
+    			} 
+    		});
+    
     setMessageSummary();
     return this;
   }
@@ -204,16 +253,21 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
   
   private void handleWebServiceURIModifyEvent()
   {
-    if (webServiceURI.getText().indexOf(':') > 0)
-      Timer.newInstance(Display.getCurrent(), this).startTimer();
+    if (webServiceURI.getText().indexOf(':') > 0) {
+      timer_ = Timer.newInstance(timer_, Display.getCurrent(), this);
+      timer_.startTimer();
+    }
     else
       handleWebServiceURI();
     statusListener_.handleEvent(null);
   }
   
-  private void handleWebServiceURI()
-  {
-    String wsURI = webServiceURI.getText();
+  private void handleWebServiceURI() {
+	  handleWebServiceURI(webServiceURI.getText());
+  }
+  
+  private void handleWebServiceURI(String wsURI)
+  {    
     if (wsURI.indexOf(':') < 0)
     {
       IFile file = uri2IFile(wsURI);
@@ -222,7 +276,7 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
     }
     if (wsURI != null && wsURI.indexOf(':') >= 0 && webServicesParser.getWebServiceEntityByURI(wsURI) == null)
     {
-      WSDLSelectionConditionCommand cmd = new WSDLSelectionConditionCommand();
+      TimedWSDLSelectionConditionCommand cmd = new TimedWSDLSelectionConditionCommand();
       cmd.setWebServicesParser(webServicesParser);
       cmd.setWebServiceURI(wsURI);
       cmd.execute(null, null);
@@ -233,7 +287,8 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
     else
       tree.setEnabled(true);
     tree.setWebServiceURI(wsURI);
-    tree.refreshTreeViewer();
+    tree.refreshTreeViewer();  
+    wsdlURI_ = wsURI;
   }
   
   public void run()
@@ -247,10 +302,45 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
     DialogResourceBrowser dialog = new DialogResourceBrowser( parent_.getShell(), null, wsFilter_);
     dialog.open();
     IResource res = dialog.getFirstSelection();
-    if( res != null ) webServiceURI.setText( res.getFullPath().toString() );
+    if( res != null )
+    {
+    	wsdlURI_ = res.getFullPath().toString();
+    	webServiceURI.setText( wsdlURI_ );    	
+    }
+    	
     statusListener_.handleEvent(null);
   }
   
+  private void setValidationInProgress(boolean validating) {
+	  stopValidationButton_.setEnabled(validating);
+  }
+  
+  private void handleStopValidationButton()
+  {
+	  IJobManager    jobManager     = Platform.getJobManager();
+	  Job[]          jobs           = jobManager.find( ValidateWSDLJob.VALIDATE_WSDL_JOB_FAMILY );
+	  ValidateWSDLJob existingValidateWSDLJob = null;
+	  
+	  if( jobs.length > 0 )
+	  {
+		  for (int i=0; i<jobs.length; i++) {
+			  existingValidateWSDLJob = (ValidateWSDLJob)jobs[i];
+
+			  if (existingValidateWSDLJob.getState() != Job.NONE) { 
+				  existingValidateWSDLJob.cancel();
+			  }
+		  }
+	  }
+	  clearValidationMessages();
+	  setValidationInProgress(false);
+  }
+  
+  private void clearValidationMessages() {
+	  msgViewer_.clearInput();
+	  validationSummaryText_.setText(" " );
+	  validationSummaryText2_.setText(" ");
+  }
+  
   public IStatus getStatus()
   {
     // Timer validation
@@ -263,33 +353,35 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
     // Validate the String representation of the Web service URI
     // For example, is it pointing to an existing resource in the workspace?
     String wsPath  = webServiceURI.getText();
-    if( wsPath == null || wsPath.length() <= 0 )
-      return StatusUtils.errorStatus( ConsumptionUIMessages.PAGE_MSG_INVALID_WEB_SERVICE_URI );
+    if( wsPath == null || wsPath.length() <= 0 ) {
+    	clearValidationMessages();
+    	return StatusUtils.errorStatus( ConsumptionUIMessages.PAGE_MSG_INVALID_WEB_SERVICE_URI );
+    }
     else if( wsPath.indexOf(':') < 0 )
     {
-      IResource res = ResourceUtils.findResource(wsPath);
-      if( res == null ) {
-    	  msgViewer_.clearInput();
-        return StatusUtils.errorStatus( NLS.bind(ConsumptionUIMessages.PAGE_MSG_NO_SUCH_FILE, new Object[] {wsPath}) );
-      }
-      else if( res.getType() != IResource.FILE ) {
-    	  msgViewer_.clearInput();
-        return StatusUtils.errorStatus( ConsumptionUIMessages.PAGE_MSG_INVALID_WEB_SERVICE_URI );
-      }
+    	IResource res = ResourceUtils.findResource(wsPath);
+    	if( res == null ) {
+    		clearValidationMessages();
+    		return StatusUtils.errorStatus( NLS.bind(ConsumptionUIMessages.PAGE_MSG_NO_SUCH_FILE, new Object[] {wsPath}) );
+    	}
+    	else if( res.getType() != IResource.FILE ) {
+    		clearValidationMessages();
+    		return StatusUtils.errorStatus( ConsumptionUIMessages.PAGE_MSG_INVALID_WEB_SERVICE_URI );
+    	}
     }
 
     
     // Validate the content of the Web service URI
     // For example, is selection a WSDL URI?
-    if (!Timer.isRunning() && tree.isEnabled())
+    if (!Timer.isRunning(timer_) && tree.isEnabled())
     {
       IStatus status = tree.getStatus();
       if (status != null)
       {
         int severity = status.getSeverity();
         if (severity == Status.ERROR || severity == Status.WARNING) {
-        	msgViewer_.clearInput();
-          return status;
+        	clearValidationMessages();
+        	return status;
         }
       }
     }
@@ -299,31 +391,36 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
         {
           String wsdlURI = iFile2URI((IFile)ResourceUtils.findResource(wsPath));
           if (webServicesParser.getWSDLDefinition(wsdlURI) == null) {
-        	  msgViewer_.clearInput();
-            return StatusUtils.errorStatus(ConsumptionUIMessages.PAGE_MSG_SELECTION_MUST_BE_WSDL );
+        	  clearValidationMessages();
+        	  return StatusUtils.errorStatus(ConsumptionUIMessages.PAGE_MSG_SELECTION_MUST_BE_WSDL );
           }
         }
     }
     
     
-    if (!Timer.isRunning()) {
+    if (!Timer.isRunning(timer_)) {
     	String wsdlURI1 = wsPath;
     	 boolean isRemote = true;
     	if (tree.isEnabled()) { // is wsil
     		wsdlURI1 = tree.getWsdlURI();
-    		if (wsdlURI1.startsWith("file:")) {
+    		if (wsdlURI1.startsWith("file:") || wsdlURI1.startsWith("platform:"))
+    		{
     			isRemote = false;
     		}
-    	} else { // is wsil
-    	
-    		if( wsPath.indexOf(':') < 0 )
-    		{      	// not remote
+    	} else {
+    		if (wsPath.indexOf(':') < 0)
+    		{
     			isRemote = false;
     			wsdlURI1 = iFile2URI((IFile)ResourceUtils.findResource(wsPath));
     		}
+    		else if (wsPath.startsWith("file:") || wsPath.startsWith("platform:"))
+    		{
+    			isRemote = false;
+    		}
     	}
   	  
       setMessageSummary();
+	  msgViewer_.clearInput();
   	  validateWSDL(wsdlURI1, isRemote);
     }
     
@@ -357,6 +454,7 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
 
 					  if (!wsdlURI.equals(existingValidateWSDLJob.getWsdlURI())) {
 						  existingValidateWSDLJob.cancel();
+						  setValidationInProgress(false);
 					  } else {						  
 						  startWSDLValidation = false;
 					  }
@@ -375,6 +473,7 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
 	  validateWSDLJob_ = new ValidateWSDLJob(wsdlURI);
 	  validateWSDLJob_.addJobChangeListener( jobChangeAdapter_ );
 	  validateWSDLJob_.schedule();
+	  setValidationInProgress(true);
   }
   
   public void updateValidationSummary(int messageSeverity)
@@ -399,6 +498,7 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
   private IFile uri2IFile(String uri)
   {
     IResource res = ResourcesPlugin.getWorkspace().getRoot().findMember(uri);
+    
     if (res instanceof IFile)
       return (IFile)res;
     else
@@ -423,13 +523,18 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
     if (initialSelection != null && !initialSelection.isEmpty())
     {
       Object object = initialSelection.getFirstElement();
-      String wsdlURI = toWsdlURI(object);
-      if (wsdlURI != null)
+      String wsdlURI = toWsdlURI(object);  
+      wsdlURI_ = wsdlURI;
+      
+      if (wsdlURI != null && webServiceURI != null)
       {
-        webServiceURI.setText(wsdlURI);
-        handleWebServiceURI();
+        handleWebServiceURI(wsdlURI);
+    	
+        webServiceURI.removeModifyListener(modifyListener_);
+        webServiceURI.setText(getObjectSelectionDisplayableString());       
+        webServiceURI.addModifyListener(modifyListener_);
       }
-    }
+    }  
   }
   
   private String toWsdlURI(Object object)
@@ -450,9 +555,13 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
   
   public IStructuredSelection getObjectSelection()
   {
-    return new StructuredSelection( 
-               new WSDLSelectionWrapper( webServicesParser,
-                                         new StructuredSelection(tree.getWsdlURI()) ) );
+	  StructuredSelection ss; 
+	  if (tree != null && tree.getWsdlURI() != null)
+		  ss = new StructuredSelection(tree.getWsdlURI());
+	  else
+		  ss = new StructuredSelection(wsdlURI_);
+	return new StructuredSelection( 
+               new WSDLSelectionWrapper( webServicesParser, ss));
   }
   
   public WebServicesParser getWebServicesParser()
@@ -467,14 +576,20 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
   
   public IProject getProject()
   {
-    String wsdlURI = tree.getWsdlURI();
+	String wsdlURI;
+
+	if (tree != null)
+       wsdlURI = tree.getWsdlURI();
+	else
+		wsdlURI = wsdlURI_;
+	
     if (wsdlURI != null)
     {
       IProject p = getProjectFromURI(wsdlURI);
       if (p!=null && p.exists())
         return p;
       
-      String wsRelPath = webServiceURI.getText();
+      String wsRelPath = wsdlURI_;
       IResource wsRes = ResourceUtils.findResource(wsRelPath);
       if (wsRes!=null && wsRes instanceof IFile)
       {
@@ -488,14 +603,20 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
   
   public String getComponentName()
   {
-    String wsdlURI = tree.getWsdlURI();
+	 
+    String wsdlURI;
+    if (tree != null)
+    	wsdlURI = tree.getWsdlURI();
+    else 
+    	wsdlURI = wsdlURI_;
+    
     if (wsdlURI != null)
     {
       String cname = getComponentNameFromURI(wsdlURI);
       if (cname!=null && cname.length()>0)
         return cname;
       
-      String wsRelPath = webServiceURI.getText();
+      String wsRelPath = wsdlURI_;
       IResource wsRes = ResourceUtils.findResource(wsRelPath);
       if (wsRes!=null && wsRes instanceof IFile)
       {
@@ -548,4 +669,48 @@ public class WSDLSelectionWidget extends AbstractObjectSelectionWidget implement
     }
     return null;    
   }
-}
\ No newline at end of file
+  
+  public String getObjectSelectionDisplayableString() 
+  {	
+    if (tree != null)
+    {
+      return transformer_.toPath(tree.getWsdlURI());
+    }
+    else
+    {
+	    return transformer_.toPath(wsdlURI_);
+    }
+	}
+  
+  public Point getWidgetSize() {	
+	  return new Point( 580, 580);  
+  }
+  
+  public boolean validate(String s) {
+	  String wsURI = s;
+	    
+	    if (wsURI.indexOf(':') < 0)
+	    {
+	      IFile file = uri2IFile(wsURI);
+	      if (file != null)
+	        wsURI = iFile2URI(file);
+	    }
+	    if (wsURI != null && wsURI.indexOf(':') >= 0 && webServicesParser.getWebServiceEntityByURI(wsURI) == null)
+	    {
+	      TimedWSDLSelectionConditionCommand cmd = new TimedWSDLSelectionConditionCommand();
+	      cmd.setWebServicesParser(webServicesParser);
+	      cmd.setWebServiceURI(wsURI);
+	      cmd.execute(null, null);
+	    }
+		
+	    // prime widget based on the string
+	    wsdlURI_ = wsURI;
+	    
+	    WebServiceEntity entity = webServicesParser.getWebServiceEntityByURI(wsURI);
+	    if (entity != null && entity.getType() == WebServiceEntity.TYPE_WSDL)
+             return true;
+	    else
+	    	return false;
+	    
+  } 
+}
