@@ -24,10 +24,11 @@ import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
+import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Path;
 import org.eclipse.emf.common.util.EList;
 import org.eclipse.emf.common.util.URI;
-import org.eclipse.emf.ecore.util.EcoreUtil;
+import org.eclipse.jem.util.UIContextDetermination;
 import org.eclipse.jem.util.emf.workbench.ProjectUtilities;
 import org.eclipse.wst.common.componentcore.ComponentCore;
 import org.eclipse.wst.common.componentcore.IEditModelHandler;
@@ -40,6 +41,8 @@ import org.eclipse.wst.common.componentcore.internal.resources.VirtualReference;
 import org.eclipse.wst.common.componentcore.internal.util.EclipseResourceAdapter;
 import org.eclipse.wst.common.componentcore.resources.IVirtualComponent;
 import org.eclipse.wst.common.componentcore.resources.IVirtualReference;
+import org.eclipse.wst.common.internal.emf.utilities.ExtendedEcoreUtil;
+import org.eclipse.wst.common.internal.emfworkbench.validateedit.IValidateEditContext;
 
 /**
  * <p>
@@ -79,7 +82,7 @@ public class StructureEdit implements IEditModelHandler {
 
 	public static final Class ADAPTER_TYPE = StructureEdit.class;
 
-	public static String MODULE_META_FILE_NAME = ".settings/.component"; //$NON-NLS-1$
+	public static String MODULE_META_FILE_NAME = ".settings/org.eclipse.wst.common.component"; //$NON-NLS-1$
 
 	private final static ComponentcoreFactory COMPONENT_FACTORY = ComponentcoreFactory.eINSTANCE;
 	private static final ComponentResource[] NO_RESOURCES = new ComponentResource[0];
@@ -237,7 +240,7 @@ public class StructureEdit implements IEditModelHandler {
 	 * @return The corresponding Eclipse IResource, if available.
 	 */
 	public static IResource getEclipseResource(ComponentResource aModuleResource) {
-		EclipseResourceAdapter eclipseResourceAdapter = (EclipseResourceAdapter) EcoreUtil.getAdapter(aModuleResource.eAdapters(), EclipseResourceAdapter.ADAPTER_TYPE);
+		EclipseResourceAdapter eclipseResourceAdapter = (EclipseResourceAdapter) ExtendedEcoreUtil.getAdapter(aModuleResource,aModuleResource.eAdapters(), EclipseResourceAdapter.ADAPTER_TYPE);
 		if (eclipseResourceAdapter != null)
 			return eclipseResourceAdapter.getEclipseResource();
 		eclipseResourceAdapter = new EclipseResourceAdapter();
@@ -350,9 +353,11 @@ public class StructureEdit implements IEditModelHandler {
 	public void save(IProgressMonitor aMonitor) {
 		if (isReadOnly)
 			throwAttemptedReadOnlyModification();
-		synchronized (structuralModel) {
-			if (!structuralModel.isDisposed())
-				structuralModel.save(aMonitor, this);
+		else if (validateEdit().isOK()) { 
+			synchronized (structuralModel) {
+				if (!structuralModel.isDisposed())
+					structuralModel.save(aMonitor, this);
+			}
 		}
 	}
 
@@ -370,9 +375,9 @@ public class StructureEdit implements IEditModelHandler {
 	public void saveIfNecessary(IProgressMonitor aMonitor) {
 		if (isReadOnly)
 			throwAttemptedReadOnlyModification();
-		synchronized (structuralModel) {
-			if (!structuralModel.isDisposed())
-				structuralModel.saveIfNecessary(aMonitor, this);
+		else if (validateEdit().isOK()) { 
+				if (!structuralModel.isDisposed())
+					structuralModel.saveIfNecessary(aMonitor, this);
 		}
 	}
 
@@ -385,20 +390,20 @@ public class StructureEdit implements IEditModelHandler {
 	 * 
 	 * @see org.eclipse.wst.common.componentcore.IEditModelHandler#dispose()
 	 */
-	public void dispose() {
-		
-		if (isStructuralModelSelfManaged) {
+	public void dispose() {	
+		if (isStructuralModelSelfManaged && structuralModel!=null) {
 			synchronized (structuralModel) {
-				if (!structuralModel.isDisposed()) {
+				if (!structuralModel.isDisposed())
 					structuralModel.releaseAccess(this);
-				}
-			}
-			
+			}	
 		}
-		if (dependentCores.size() > 0) {
+		if (dependentCores!=null && dependentCores.size() > 0) {
 			synchronized (dependentCores) {
-				for (Iterator cores = dependentCores.values().iterator(); cores.hasNext();)
-					((StructureEdit) cores.next()).dispose();
+				for (Iterator cores = dependentCores.values().iterator(); cores.hasNext();) {
+					StructureEdit core = (StructureEdit) cores.next();
+					if (core != null)
+						core.dispose();
+				}
 			}
 		}
 	}
@@ -433,8 +438,10 @@ public class StructureEdit implements IEditModelHandler {
 	 */
 	public ProjectComponents getComponentModelRoot() {
 		ProjectComponents comps = null;
-		if (!structuralModel.isDisposed())
-			comps = (ProjectComponents) structuralModel.getPrimaryRootObject();
+		synchronized(structuralModel){
+			if (!structuralModel.isDisposed())
+				comps = (ProjectComponents) structuralModel.getPrimaryRootObject();
+		}
 		return comps;
 	}
 
@@ -763,15 +770,19 @@ public class StructureEdit implements IEditModelHandler {
 	 * @see WorkbenchComponent#getHandle()
 	 */
 	public WorkbenchComponent findComponentByURI(URI aModuleURI) throws UnresolveableURIException {
+		if (aModuleURI == null)
+			return null;
 		if(aModuleURI.scheme() == null && aModuleURI.segmentCount() == 1)
 			return getComponent();
 		ModuleURIUtil.ensureValidFullyQualifiedModuleURI(aModuleURI);
 		String projectName = aModuleURI.segment(ModuleURIUtil.ModuleURI.PROJECT_NAME_INDX);
 		/* Accessing a local module */
-		if (getProject().getName().equals(projectName)) {
+		if (getProject()!=null && getProject().getName().equals(projectName))
 			return getComponent();
-		}
-		return getDependentModuleCore(aModuleURI).getComponent();
+		StructureEdit dependentCore = getDependentModuleCore(aModuleURI);
+		if (dependentCore != null)
+			return dependentCore.getComponent();
+		return null;
 	}
 
 	/**
@@ -902,8 +913,9 @@ public class StructureEdit implements IEditModelHandler {
 	 * 
 	 */
 	public WorkbenchComponent getComponent() {
-		if (getWorkbenchModules().length > 0)
-			return getWorkbenchModules()[0];
+		WorkbenchComponent[] comps = getWorkbenchModules();
+		if (comps.length > 0)
+			return comps[0];
 		return null;
 	}
 
@@ -915,4 +927,16 @@ public class StructureEdit implements IEditModelHandler {
 	protected IProject getProject() {
 		return aProject;
 	}
+	
+	public ModuleStructuralModel getModuleStructuralModel() {
+		return structuralModel;
+	}
+	
+	/**
+	 * Validate edit for resource state
+	 */
+	protected IStatus validateEdit() {
+		IValidateEditContext validator = (IValidateEditContext) UIContextDetermination.createInstance(IValidateEditContext.CLASS_KEY);
+		return validator.validateState(getModuleStructuralModel());
+	}
 }
