@@ -22,9 +22,12 @@ import org.eclipse.jface.dialogs.Dialog;
 import org.eclipse.jface.dialogs.IMessageProvider;
 import org.eclipse.jface.viewers.DoubleClickEvent;
 import org.eclipse.jface.viewers.IDoubleClickListener;
+import org.eclipse.jface.viewers.ILabelProvider;
+import org.eclipse.jface.viewers.ILabelProviderListener;
 import org.eclipse.jface.viewers.ISelectionChangedListener;
 import org.eclipse.jface.viewers.IStructuredSelection;
 import org.eclipse.jface.viewers.ITreeContentProvider;
+import org.eclipse.jface.viewers.LabelProviderChangedEvent;
 import org.eclipse.jface.viewers.SelectionChangedEvent;
 import org.eclipse.jface.viewers.TreeViewer;
 import org.eclipse.jface.viewers.Viewer;
@@ -43,6 +46,7 @@ import org.eclipse.ui.PlatformUI;
 import org.eclipse.wst.server.core.*;
 import org.eclipse.wst.server.ui.ServerUICore;
 import org.eclipse.wst.server.ui.internal.*;
+import org.eclipse.wst.server.ui.internal.view.servers.ModuleServer;
 import org.eclipse.wst.server.ui.wizard.IWizardHandle;
 /**
  * A wizard page used to add and remove modules.
@@ -71,8 +75,13 @@ public class ModifyModulesComposite extends Composite {
 	protected Button remove, removeAll;
 
 	protected TaskModel taskModel;
-	protected IModule newModule;
-	protected IModule origNewModule;
+
+	// a module that must be kept on the server
+	protected IModule requiredModule;
+
+	// the parent modules of the above modules. at least one of these modules
+	// must be kept on the server
+	protected IModule[] requiredModules;
 
 	protected Map errorMap;
 
@@ -86,7 +95,8 @@ public class ModifyModulesComposite extends Composite {
 		}
 		
 		public Object[] getChildren(Object parentElement) {
-			IModule[] parent = (IModule[]) parentElement;
+			ModuleServer ms = (ModuleServer) parentElement;
+			IModule[] parent = ms.module;
 			IModule[] children = (IModule[]) childModuleMap.get(new ChildModuleMapKey(parent));
 			
 			List list = new ArrayList();
@@ -101,19 +111,24 @@ public class ModifyModulesComposite extends Composite {
 					IModule[] module2 = new IModule[size2 + 1];
 					System.arraycopy(parent, 0, module2, 0, size2);
 					module2[size2] = child;
-					list.add(module2);
+					list.add(new ModuleServer(null, module2));
 				}
 			}
 			return list.toArray();
 		}
 
 		public Object getParent(Object element) {
-			IModule[] child = (IModule[]) element;
-			return (IModule[]) parentModuleMap.get(child);
+			ModuleServer ms = (ModuleServer) element;
+			IModule[] child = ms.module;
+			IModule[] modules2 = (IModule[]) parentModuleMap.get(child);
+			if (modules2 == null)
+				return null;
+			return new ModuleServer(null, modules2);
 		}
 
 		public boolean hasChildren(Object element) {
-			IModule[] parent = (IModule[]) element;
+			ModuleServer ms = (ModuleServer) element;
+			IModule[] parent = ms.module;
 			IModule[] children = (IModule[]) childModuleMap.get(new ChildModuleMapKey(parent));
 			return (children != null && children.length > 0);
 		}
@@ -125,7 +140,7 @@ public class ModifyModulesComposite extends Composite {
 			Iterator iterator = modules.iterator();
 			while (iterator.hasNext()) {
 				IModule module = (IModule) iterator.next();
-				list.add(new IModule[] { module });
+				list.add(new ModuleServer(null, new IModule[] { module }));
 			}
 			return list.toArray();
 		}
@@ -137,7 +152,7 @@ public class ModifyModulesComposite extends Composite {
 			Iterator iterator = deployed.iterator();
 			while (iterator.hasNext()) {
 				IModule module = (IModule) iterator.next();
-				list.add(new IModule[] { module });
+				list.add(new ModuleServer(null, new IModule[] { module }));
 			}
 			return list.toArray();
 		}
@@ -148,7 +163,7 @@ public class ModifyModulesComposite extends Composite {
 	 * ChildMapModuleKey
 	 */
 	protected class ChildModuleMapKey {
-		private IModule[] moduleTree;
+		protected IModule[] moduleTree;
 		
 		protected ChildModuleMapKey(IModule curModule) {
 			if (curModule != null) {
@@ -200,15 +215,15 @@ public class ModifyModulesComposite extends Composite {
 	public ModifyModulesComposite(Composite parent, IWizardHandle wizard, IModule module) {
 		super(parent, SWT.NONE);
 		this.wizard = wizard;
-		origNewModule = module;
-			
+		requiredModule = module;
+		
 		wizard.setTitle(Messages.wizModuleTitle);
 		wizard.setDescription(Messages.wizModuleDescription);
 		wizard.setImageDescriptor(ImageResource.getImageDescriptor(ImageResource.IMG_WIZBAN_SELECT_SERVER));
 		
 		createControl();
 	}
-	
+
 	public void setServer(IServerAttributes server) {
 		if (server == this.server)
 			return;
@@ -234,25 +249,24 @@ public class ModifyModulesComposite extends Composite {
 		}
 		
 		// add new module
-		newModule = null;
+		requiredModules = null;
 		errorMap = new HashMap();
-		if (origNewModule != null) {
+		if (requiredModule != null) {
 			try {
-				IModule[] parents = server.getRootModules(origNewModule, null);
+				IModule[] parents = server.getRootModules(requiredModule, null);
 				if (parents != null && parents.length > 0)
-					newModule = parents[0];
+					requiredModules = parents;
 				else
-					newModule = origNewModule;
+					requiredModules = new IModule[] { requiredModule };
 			} catch (CoreException ce) {
-				errorMap.put(newModule, ce.getStatus());
-				newModule = null;
+				// ignore
+				//errorMap.put(newModule, ce.getStatus());
 			} catch (Exception e) {
 				Trace.trace(Trace.WARNING, "Could not find root module", e);
-				newModule = null;
 			}
 		}
-		if (newModule != null && !deployed.contains(newModule))
-			deployed.add(newModule);
+		if (requiredModules != null && !deployed.contains(requiredModules[0]))
+			deployed.add(requiredModules[0]);
 
 		// get remaining modules
 		IModule[] modules2 = ServerUtil.getModules(server.getServerType().getRuntimeType().getModuleTypes());
@@ -294,7 +308,7 @@ public class ModifyModulesComposite extends Composite {
 				// ignore
 			}
 		}
-
+		
 		iterator = modules.iterator();
 		while (iterator.hasNext()) {
 			IModule module = (IModule) iterator.next();
@@ -307,6 +321,37 @@ public class ModifyModulesComposite extends Composite {
 			}
 		}
 		
+		// get children recursively one level
+		// put child elements into a different list to avoid concurrent modifications
+		iterator = childModuleMap.keySet().iterator();
+		List list = new ArrayList();
+		while (iterator.hasNext()) {
+			list.add(iterator.next());
+		}
+		
+		iterator = list.iterator();
+		while (iterator.hasNext()) {
+			ChildModuleMapKey key = (ChildModuleMapKey) iterator.next();
+			IModule[] children0 = (IModule[]) childModuleMap.get(key);
+			if (children0 != null) {
+				int size = children0.length;
+				for (int i = 0; i < size; i++) {
+					int size2 = key.moduleTree.length;
+					IModule[] module2 = new IModule[size2 + 1];
+					System.arraycopy(key.moduleTree, 0, module2, 0, size2);
+					module2[size2] = children0[i];
+					
+					try {
+						IModule[] children = server.getChildModules(module2, null);
+						if (children != null && children.length > 0)
+							childModuleMap.put(new ChildModuleMapKey(module2), children);
+					} catch (Exception e) {
+						// ignore
+					}
+				}
+			}
+		}
+		
 		if (availableTreeViewer != null)
 			Display.getDefault().syncExec(new Runnable() {
 				public void run() {
@@ -352,7 +397,7 @@ public class ModifyModulesComposite extends Composite {
 		
 		label = new Label(this, SWT.NONE);
 		label.setText(Messages.wizModuleDeployedList);
-
+		
 		Tree availableTree = new Tree(this, SWT.BORDER);
 		data = new GridData(GridData.FILL_BOTH);
 		data.heightHint = 200;
@@ -360,7 +405,20 @@ public class ModifyModulesComposite extends Composite {
 		availableTree.setLayoutData(data);
 		
 		availableTreeViewer = new TreeViewer(availableTree);
-		availableTreeViewer.setLabelProvider(ServerUICore.getLabelProvider());
+		ILabelProvider labelProvider = ServerUICore.getLabelProvider();
+		labelProvider.addListener(new ILabelProviderListener() {
+			public void labelProviderChanged(LabelProviderChangedEvent event) {
+				Object[] obj = event.getElements();
+				if (obj == null)
+					availableTreeViewer.refresh(true);
+				else {
+					int size = obj.length;
+					for (int i = 0; i < size; i++)
+						availableTreeViewer.refresh(obj[i], true);
+				}
+			}
+		});
+		availableTreeViewer.setLabelProvider(labelProvider);
 		availableTreeViewer.setContentProvider(new AvailableContentProvider());
 		availableTreeViewer.setSorter(new ViewerSorter());
 		availableTreeViewer.setInput("root");
@@ -377,19 +435,19 @@ public class ModifyModulesComposite extends Composite {
 					add(false);
 			}
 		});
-
+		
 		// slosh buttons
 		Composite comp = new Composite(this, SWT.NONE);
 		data = new GridData(GridData.FILL_BOTH);
 		data.widthHint = 120;
 		comp.setLayoutData(data);
-
+		
 		layout = new GridLayout();
 		layout.marginWidth = 5;
 		layout.marginHeight = 25;
 		layout.verticalSpacing = 20;
 		comp.setLayout(layout);
-
+		
 		add = new Button(comp, SWT.PUSH);
 		add.setText(Messages.wizModuleAdd);
 		add.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
@@ -435,7 +493,20 @@ public class ModifyModulesComposite extends Composite {
 		deployedTree.setLayoutData(data);
 		
 		deployedTreeViewer = new TreeViewer(deployedTree);
-		deployedTreeViewer.setLabelProvider(ServerUICore.getLabelProvider());
+		labelProvider = ServerUICore.getLabelProvider();
+		labelProvider.addListener(new ILabelProviderListener() {
+			public void labelProviderChanged(LabelProviderChangedEvent event) {
+				Object[] obj = event.getElements();
+				if (obj == null)
+					deployedTreeViewer.refresh(true);
+				else {
+					int size = obj.length;
+					for (int i = 0; i < size; i++)
+						deployedTreeViewer.refresh(obj[i], true);
+				}
+			}
+		});
+		deployedTreeViewer.setLabelProvider(labelProvider);
 		deployedTreeViewer.setContentProvider(new DeployedContentProvider());
 		deployedTreeViewer.setSorter(new ViewerSorter());
 		deployedTreeViewer.setInput("root");
@@ -461,17 +532,18 @@ public class ModifyModulesComposite extends Composite {
 
 	protected IModule getAvailableSelection() {
 		IStructuredSelection sel = (IStructuredSelection) availableTreeViewer.getSelection();
-		return getModule((IModule[]) sel.getFirstElement());
+		return getModule((ModuleServer) sel.getFirstElement());
 	}
 
 	protected IModule getDeployedSelection() {
 		IStructuredSelection sel = (IStructuredSelection) deployedTreeViewer.getSelection();
-		return getModule((IModule[]) sel.getFirstElement());
+		return getModule((ModuleServer) sel.getFirstElement());
 	}
 
-	protected static IModule getModule(IModule[] modules2) {
-		if (modules2 == null)
+	protected static IModule getModule(ModuleServer ms) {
+		if (ms == null)
 			return null;
+		IModule[] modules2 = ms.module;
 		return modules2[modules2.length - 1];
 	}
 
@@ -497,21 +569,36 @@ public class ModifyModulesComposite extends Composite {
 				// ignore
 			}
 		}
+		
 		add.setEnabled(enabled);
 		addAll.setEnabled(modules.size() > 0);
 		
 		enabled = false;
 		module = getDeployedSelection();
-		if (module != null) {
+		if (module != null && deployed.contains(module)) {
 			try {
-				if (deployed.contains(module) && !module.equals(newModule))
+				int count = 0;
+				boolean found = false;
+				if (requiredModules != null) {
+					// count how many of newModule are deployed
+					int size = requiredModules.length;
+					for (int i = 0; i < size; i++) {
+						if (deployed.contains(requiredModules[i]))
+							count++;
+						if (module.equals(requiredModules[i]))
+							found = true;
+					}
+					if (count > 1 || !found)
+						enabled = true;
+				} else {
 					enabled = true;
+				}
 			} catch (Exception e) {
 				// ignore
 			}
 		}
 		remove.setEnabled(enabled);
-		if (newModule == null)
+		if (requiredModules == null)
 			removeAll.setEnabled(deployed.size() > 0);
 		else
 			removeAll.setEnabled(deployed.size() > 1);
@@ -529,8 +616,23 @@ public class ModifyModulesComposite extends Composite {
 
 	protected void remove(boolean all) {
 		if (all) {
-			IModule[] modules2 = new IModule[deployed.size()];
-			deployed.toArray(modules2);
+			// pick one of the required modules to keep
+			IModule keep = null;
+			if (requiredModules != null) {
+				int size2 = requiredModules.length;
+				for (int i = 0; i < size2; i++) {
+					if (keep == null && deployed.contains(requiredModules[i]))
+						keep = requiredModules[i];
+				}
+			}
+			
+			List list = new ArrayList();
+			list.addAll(deployed);
+			list.remove(keep);
+			
+			IModule[] modules2 = new IModule[list.size()];
+			list.toArray(modules2);
+			
 			moveAll(modules2, false);
 		} else
 			moveAll(new IModule[] { getDeployedSelection() }, false);
@@ -546,14 +648,14 @@ public class ModifyModulesComposite extends Composite {
 			if (status == null && !list.contains(mods[i]))
 				list.add(mods[i]);
 		}
-
+		
 		Iterator iterator = list.iterator();
 		while (iterator.hasNext()) {
 			IModule module = (IModule) iterator.next();
 			if (add2) {
 				modules.remove(module);
 				deployed.add(module);
-			} else if (!module.equals(newModule)) {
+			} else {
 				modules.add(module);
 				deployed.remove(module);
 			}
@@ -564,7 +666,7 @@ public class ModifyModulesComposite extends Composite {
 
 		setEnablement();
 	}
-	
+
 	protected void updateTaskModel() {
 		if (taskModel == null)
 			return;
@@ -583,7 +685,7 @@ public class ModifyModulesComposite extends Composite {
 		}
 		return list;
 	}
-	
+
 	public List getModulesToAdd() {
 		List list = new ArrayList();
 		Iterator iterator = deployed.iterator();
