@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2004 IBM Corporation and others.
+ * Copyright (c) 2004, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,6 +10,9 @@
  * yyyymmdd bug      Email and other contact information
  * -------- -------- -----------------------------------------------------------
  * 20060131   125777 jesper@selskabet.org - Jesper S Moller
+ * 20060222   118019 andyzhai@ca.ibm.com - Andy Zhai
+ * 20060222   128564 jesper@selskabet.org - Jesper S Moller
+ * 20060823    99034 makandre@ca.ibm.com - Andrew Mak, WSE support for basic-authenticating firewalls
  *******************************************************************************/
 package org.eclipse.wst.ws.internal.explorer.platform.wsdl.transport;
 
@@ -34,12 +37,16 @@ public class HTTPTransport
 {
   private final String SYS_PROP_HTTPS_PROXY_HOST = "https.proxyHost";
   private final String SYS_PROP_HTTPS_PROXY_PORT = "https.proxyPort";
+  private final String SYS_PROP_HTTPS_NON_PROXY_HOSTS = "https.nonProxyHosts";
   private final String SYS_PROP_HTTP_PROXY_HOST = "http.proxyHost";
   private final String SYS_PROP_HTTP_PROXY_PORT = "http.proxyPort";
   private final String SYS_PROP_HTTP_PROXY_USER_NAME = "http.proxyUserName";
   private final String SYS_PROP_HTTP_PROXY_PASSWORD = "http.proxyPassword";
-
+  private final String SYS_PROP_HTTP_NON_PROXY_HOSTS = "http.nonProxyHosts";
+  
   private final String HTTP_METHOD = "POST";
+  private final String HTTP_CONNECT = "CONNECT";
+  private final String HTTP_VERSION_1_0 = "HTTP/1.0";
   private final String HTTP_VERSION = "HTTP/1.1";
   private final String HTTP_HEADER_SOAP_ACTION = "SOAPAction";
   public static final String HTTP_HEADER_AUTH = "Authorization";
@@ -58,7 +65,7 @@ public class HTTPTransport
   private final String HTTP_HEADER_CONNECTION = "Connection";
   
   private final int HTTP_CODE_CONTINUE = 100;
-  //private final int HTTP_CODE_OK = 200;
+  private final int HTTP_CODE_OK = 200;
   private final int HTTP_CODE_EXCEPTION = 300;
 
   private final String IBM_WEB_SERVICES_EXPLORER = "IBM Web Services Explorer";
@@ -84,6 +91,7 @@ public class HTTPTransport
   private final int DEFAULT_HTTPS_PORT = 443;
   private final String DEFAULT_SOAP_ENCODING = "UTF-8";
   private final String DEFAULT_HTTP_HEADER_ENCODING = "iso-8859-1";
+  private final boolean DEFAULT_CASE_SENSITIVE_FOR_HOST_NAME = false;
 
   private String httpBasicAuthUsername;
   private String httpBasicAuthPassword;
@@ -164,9 +172,9 @@ public class HTTPTransport
     return getHTTPHeader(HTTP_HEADER_CONTENT_TYPE, CONTENT_TYPE_VALUE);
   }
   
-  private String getContentLength(String payload)
+  private String getContentLength(byte[] payloadAsBytes)
   {
-    return getHTTPHeader(HTTP_HEADER_CONTENT_LENGTH, String.valueOf(payload.length()));
+    return getHTTPHeader(HTTP_HEADER_CONTENT_LENGTH, String.valueOf(payloadAsBytes.length));
   }
 
   private String getSOAPAction(String soapAction)
@@ -288,11 +296,13 @@ public class HTTPTransport
 
   public void send(URL url, String soapAction, String payload) throws UnknownHostException, IOException
   {
+    byte[] payloadAsUTF8 = payload.getBytes(DEFAULT_SOAP_ENCODING);
+       
     StringBuffer httpHeader = new StringBuffer();
     httpHeader.append(getMethod(url));
     httpHeader.append(getHost(url));
     httpHeader.append(getContentType());
-    httpHeader.append(getContentLength(payload));
+    httpHeader.append(getContentLength(payloadAsUTF8));
     httpHeader.append(getAccept());
     httpHeader.append(getUserAgent());
     httpHeader.append(getCacheControl());
@@ -308,7 +318,7 @@ public class HTTPTransport
     InputStream is = socket.getInputStream();
     OutputStream os = socket.getOutputStream();
     os.write(httpHeader.toString().getBytes(DEFAULT_HTTP_HEADER_ENCODING));
-    os.write(payload.getBytes(DEFAULT_SOAP_ENCODING));
+    os.write(payloadAsUTF8);
     os.flush();
     httpResponse = new HTTPResponse();
     readHTTPResponseHeader(is, httpResponse);
@@ -465,6 +475,61 @@ public class HTTPTransport
     return headers;
   }
 
+  /**
+   * Builds the first line of a connection request to the proxy server
+   * 
+   * @param url The URL that we want to ultimately connect to.
+   * @return A string of the form "CONNECT &lt;hostname&gt;:&lt;port&gt; HTTP/1.0".
+   */
+  private StringBuffer getConnectMethod(URL url) {
+	
+	  StringBuffer sb = new StringBuffer(HTTP_CONNECT);
+	  sb.append(SPACE);	  
+	  sb.append(url.getHost());
+	  sb.append(COLON);
+	  sb.append(url.getPort());
+	  sb.append(SPACE);
+	  sb.append(HTTP_VERSION_1_0);
+	  sb.append(NEW_LINE);
+	  return sb;
+  }
+  
+  /**
+   * Construct a socket to the proxy server which be used to tunnel through.
+   * 
+   * @param url The URL that we want to ultimately connect to.
+   * @param proxyHost The proxy host.
+   * @param proxyPort The proxy port.
+   * @return A connected socket to the proxy server. 
+   * @throws IOException 
+   */
+  private Socket buildTunnelSocket(URL url, String proxyHost, int proxyPort) throws IOException {
+
+	  StringBuffer httpHeader = new StringBuffer();
+	  httpHeader.append(getConnectMethod(url));
+	  httpHeader.append(getProxyAuthentication());       
+	  httpHeader.append(NEW_LINE);
+    
+	  Socket tunnel = new Socket(proxyHost, proxyPort);
+    
+	  InputStream  is = tunnel.getInputStream();
+	  OutputStream os = tunnel.getOutputStream();
+    
+	  os.write(httpHeader.toString().getBytes(DEFAULT_HTTP_HEADER_ENCODING));        
+	  os.flush();
+    
+	  HTTPResponse httpResponse = new HTTPResponse();
+	  readHTTPResponseHeader(is, httpResponse);
+    
+	  int code = httpResponse.getStatusCode();
+
+	  // ensure we are successfully connected to the proxy
+	  if (code != HTTP_CODE_OK)
+		  throw new HTTPException(code, httpResponse.getStatusMessage(), httpResponse.getHeaders());
+	  
+	  return tunnel;
+  }
+  
   private Socket buildSocket(URL url) throws UnknownHostException, IOException
   {
     Socket s = null;
@@ -472,16 +537,22 @@ public class HTTPTransport
     int port = url.getPort();
     String proxyHost = System.getProperty(SYS_PROP_HTTP_PROXY_HOST);
     int proxyPort = Integer.getInteger(SYS_PROP_HTTP_PROXY_PORT, DEFAULT_HTTP_PORT).intValue();
-//    String proxyUserName = System.getProperty(SYS_PROP_HTTP_PROXY_USER_NAME);
-//    String proxyPassword = System.getProperty(SYS_PROP_HTTP_PROXY_PASSWORD);
+    
+    String nonProxyHosts = System.getProperty(SYS_PROP_HTTP_NON_PROXY_HOSTS);
+
+    //  String proxyUserName = System.getProperty(SYS_PROP_HTTP_PROXY_USER_NAME);
+    //  String proxyPassword = System.getProperty(SYS_PROP_HTTP_PROXY_PASSWORD);
     if (url.getProtocol().equalsIgnoreCase(HTTPS))
     {
       proxyHost = System.getProperty(SYS_PROP_HTTPS_PROXY_HOST);
       proxyPort = Integer.getInteger(SYS_PROP_HTTPS_PROXY_PORT, DEFAULT_HTTPS_PORT).intValue();
-      if (proxyHost != null && proxyHost.length() > 0)
+      nonProxyHosts = System.getProperty(SYS_PROP_HTTPS_NON_PROXY_HOSTS);
+
+      if (proxyHost != null && proxyHost.length() > 0 && !isHostInNonProxyHosts(host, nonProxyHosts, DEFAULT_CASE_SENSITIVE_FOR_HOST_NAME))
       {
-        // TODO:
         // SSL with proxy server
+        Socket tunnel = buildTunnelSocket(url, proxyHost, proxyPort);       
+        s = ((SSLSocketFactory) SSLSocketFactory.getDefault()).createSocket(tunnel, host, port, true);    	  
       }
       else
         s = SSLSocketFactory.getDefault().createSocket(host, (port > 0 ? port : DEFAULT_HTTPS_PORT));
@@ -492,10 +563,54 @@ public class HTTPTransport
       // as demonstrated in the following (original) line of code:
       //  s = SSLUtils.buildSSLSocket(host, (port > 0 ? port : DEFAULT_HTTPS_PORT), proxyHost, proxyPort, proxyUserName, proxyPassword);
     }
-    else if (proxyHost != null && proxyHost.length() > 0)
+    else if (proxyHost != null && proxyHost.length() > 0 && !isHostInNonProxyHosts(host, nonProxyHosts, DEFAULT_CASE_SENSITIVE_FOR_HOST_NAME))
       s = new Socket(proxyHost, proxyPort);
     else
       s = new Socket(host, (port > 0 ? port : DEFAULT_HTTP_PORT));
     return s;
   }
+  
+  private boolean isHostInNonProxyHosts(String host, String nonProxyHosts, boolean caseSensitive)
+  {
+  	if (caseSensitive) return host.matches(createPatternFromString(nonProxyHosts));
+  	else return host.toLowerCase().matches(createPatternFromString(nonProxyHosts.toLowerCase()));  
+  }
+  
+  /*
+   * This method is used to generate a valid regular expression for a 
+   * normal java String used in the proxy mechanism. 
+   * For example, the http.nonProxyHosts contains following String: 
+   * "192.168.2.*|localhost|*.ibm.com" . It would be 
+   * transformed into: "192\.168\.2\.\w*|localhost|\w*\.ibm\.com"
+   * Thus, following host string would match above pattern:
+   * 192.168.2.5 192.168.2. 192.168.2.666 192.168.2.w
+   * localhost
+   * torolab.ibm.com .ibm.com 123.ibm.com
+   * 
+   * Two transformations are done:
+   * 1. replace all "." into "\." As in regular expression, '.' represents 
+   *    any charater.  "\.' represents the real character '.'
+   * 2. In order to get the real meaning of "*" used in property 
+   *    http.nonProxyHosts, "\w*" is used to replace "*"
+   *    "\w" represent a word character: [a-zA-Z_0-9]
+   *    
+   * Restriction:
+   * The validity of address is not checked. 
+   * (192.168.2.555 and 192.168.2.com are OK)
+   * 
+   * TODO check whether * occurs in address or hostname.
+   * if it occuus in address representation, replace "*" with "\d*"
+   * and check: value < 256 ?
+   */
+  private String createPatternFromString(String str) 
+  {
+    /* This is the same as following more understandable way:
+	 * return str.replace(".", "\\.").replace("*", "\\w*");
+	 * But, replace(CharSequence target, CharSequence replacement) can only be 
+	 * supported after j2se 1.5, on the other hand, 
+	 * replaceAll(String regex, String replacement) can be supported before 
+	 * j2se 1.5.
+	 */
+    return str == null ? null : str.replaceAll("\\.", "\\.").replaceAll("\\*", "\\w*");
+  }
 }
\ No newline at end of file
