@@ -1,5 +1,5 @@
 /*****************************************************************************
- * Copyright (c) 2004 IBM Corporation and others. All rights reserved. This
+ * Copyright (c) 2004, 2007 IBM Corporation and others. All rights reserved. This
  * program and the accompanying materials are made available under the terms
  * of the Eclipse Public License v1.0 which accompanies this distribution, and
  * is available at http://www.eclipse.org/legal/epl-v10.html
@@ -40,11 +40,11 @@ import org.eclipse.ui.PartInitException;
 import org.eclipse.ui.PlatformUI;
 import org.eclipse.ui.ide.IDE;
 import org.eclipse.ui.ide.IGotoMarker;
+import org.eclipse.ui.part.MultiPageEditorActionBarContributor;
+import org.eclipse.ui.part.MultiPageEditorPart;
+import org.eclipse.ui.part.MultiPageEditorSite;
 import org.eclipse.ui.part.MultiPageSelectionProvider;
 import org.eclipse.ui.progress.UIJob;
-import org.eclipse.wst.common.ui.provisional.editors.PostMultiPageEditorSite;
-import org.eclipse.wst.common.ui.provisional.editors.PostMultiPageSelectionProvider;
-import org.eclipse.wst.common.ui.provisional.editors.PostSelectionMultiPageEditorPart;
 import org.eclipse.wst.sse.core.internal.provisional.IndexedRegion;
 import org.eclipse.wst.sse.core.internal.provisional.text.ITextRegion;
 import org.eclipse.wst.sse.ui.StructuredTextEditor;
@@ -53,7 +53,7 @@ import org.eclipse.wst.xml.core.internal.provisional.contenttype.ContentTypeIdFo
 import org.eclipse.wst.xml.ui.internal.Logger;
 import org.eclipse.wst.xml.ui.internal.XMLUIPlugin;
 
-public class XMLMultiPageEditorPart extends PostSelectionMultiPageEditorPart {
+public class XMLMultiPageEditorPart extends MultiPageEditorPart {
 
 	/**
 	 * Internal part activation listener, copied from AbstractTextEditor
@@ -364,7 +364,7 @@ public class XMLMultiPageEditorPart extends PostSelectionMultiPageEditorPart {
 		if (fDesignViewer.getSelectionProvider() instanceof IPostSelectionProvider) {
 			((IPostSelectionProvider) fDesignViewer.getSelectionProvider()).addPostSelectionChangedListener(new ISelectionChangedListener() {
 				public void selectionChanged(SelectionChangedEvent event) {
-					((PostMultiPageSelectionProvider) getSite().getSelectionProvider()).firePostSelectionChanged(event);
+					((MultiPageSelectionProvider) getSite().getSelectionProvider()).firePostSelectionChanged(event);
 				}
 			});
 		}
@@ -505,6 +505,14 @@ public class XMLMultiPageEditorPart extends PostSelectionMultiPageEditorPart {
 			addSourcePage();
 			connectDesignPage();
 
+			// set the active editor in the action bar contributor first
+			// before setactivepage calls action bar contributor's
+			// setactivepage (bug141013 - remove when bug151488 is fixed)
+			IEditorActionBarContributor contributor = getEditorSite().getActionBarContributor();
+			if (contributor instanceof MultiPageEditorActionBarContributor) {
+				((MultiPageEditorActionBarContributor) contributor).setActiveEditor(this);
+			}
+
 			int activePageIndex = getPreferenceStore().getInt(IXMLPreferenceNames.LAST_ACTIVE_PAGE);
 			if (activePageIndex >= 0 && activePageIndex < getPageCount()) {
 				setActivePage(activePageIndex);
@@ -525,7 +533,7 @@ public class XMLMultiPageEditorPart extends PostSelectionMultiPageEditorPart {
 	protected IEditorSite createSite(IEditorPart editor) {
 		IEditorSite site = null;
 		if (editor == fTextEditor) {
-			site = new PostMultiPageEditorSite(this, editor) {
+			site = new MultiPageEditorSite(this, editor) {
 				/**
 				 * @see org.eclipse.ui.part.MultiPageEditorSite#getActionBarContributor()
 				 */
@@ -618,6 +626,16 @@ public class XMLMultiPageEditorPart extends PostSelectionMultiPageEditorPart {
 	 */
 	public void doSaveAs() {
 		fTextEditor.doSaveAs();
+		/**
+		 * TEMPORARY WORKAROUND, the property change listener should really
+		 * handle this
+		 * 
+		 * Update the design viewer since the editor input would have changed
+		 * to the new file.
+		 */
+		if (fDesignViewer != null) {
+			fDesignViewer.setDocument(getDocument());
+		}
 	}
 
 	/*
@@ -661,7 +679,7 @@ public class XMLMultiPageEditorPart extends PostSelectionMultiPageEditorPart {
 	private IPreferenceStore getPreferenceStore() {
 		return XMLUIPlugin.getDefault().getPreferenceStore();
 	}
-	
+
 	StructuredTextEditor getTextEditor() {
 		return fTextEditor;
 	}
@@ -741,11 +759,13 @@ public class XMLMultiPageEditorPart extends PostSelectionMultiPageEditorPart {
 		saveLastActivePageIndex(newPageIndex);
 
 		if (newPageIndex == fDesignPageIndex) {
+			// design page isn't an IEditorPart, therefore we have to send
+			// selection changes ourselves
 			ISelectionProvider selectionProvider = fDesignViewer.getSelectionProvider();
 			if (selectionProvider != null) {
 				SelectionChangedEvent event = new SelectionChangedEvent(selectionProvider, selectionProvider.getSelection());
 				((MultiPageSelectionProvider) getSite().getSelectionProvider()).fireSelectionChanged(event);
-				((PostMultiPageSelectionProvider) getSite().getSelectionProvider()).firePostSelectionChanged(event);
+				((MultiPageSelectionProvider) getSite().getSelectionProvider()).firePostSelectionChanged(event);
 			}
 		}
 	}
@@ -761,12 +781,16 @@ public class XMLMultiPageEditorPart extends PostSelectionMultiPageEditorPart {
 	 * @see org.eclipse.ui.part.EditorPart#setInput(org.eclipse.ui.IEditorInput)
 	 */
 	protected void setInput(IEditorInput input) {
-		// If driven from the Source page, it's "model" may not be up to date
-		// with the input just yet. We'll rely on later notification from the
-		// TextViewer to set us straight
+		/*
+		 * If driven from the Source page, it's "model" may not be up to date
+		 * with (or even exist for) the input just yet. Later notification
+		 * from the TextViewer could set us straight, although it's not
+		 * guaranteed to happen after the model has been created.
+		 */
 		super.setInput(input);
-		if (fDesignViewer != null)
+		if (fDesignViewer != null) {
 			fDesignViewer.setDocument(getDocument());
+		}
 		setPartName(input.getName());
 	}
 }
\ No newline at end of file
